<?php

/**
 * @package     Joomla - > Site and Administrator payment info
 * @subpackage  com_Digicom
 * @subpackage 	joomina_Zarinpal
 * @copyright   joomina team => https://joomina.com
 * @copyright   Copyright (C) 20016 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

require_once(dirname(__FILE__) . '/digiirankish/helper.php');
if (!class_exists('checkHack')) {
	require_once(dirname(__FILE__) . '/digiirankish/irankish_inputcheck.php');
}

class plgDigiCom_Paydigiirankish extends JPlugin
{
	protected $autoloadLanguage = true;

	function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);
		$this->responseStatus = array(
			'Completed' => 'A',
			'Pending' 	=> 'P',
			'Failed' 		=> 'P',
			'Denied' 		=> 'P',
			'Refunded'	=> 'RF'
		);
	}

	public function onDigicomSidebarMenuItem()
	{
		$pluginid = $this->getPluginId('digiirankish', 'digicom_pay', 'plugin');
		$params 	= $this->params;
		$link 		= JRoute::_("index.php?option=com_plugins&client_id=0&task=plugin.edit&extension_id=" . $pluginid);
		return '<a target="_blank" href="' . $link . '" title="digiirankish" id="plugin-' . $pluginid . '">' . 'digiirankish' . '</a>';
	}

	function getPluginId($element, $folder, $type)
	{
		$db = JFactory::getDBO();
		$query = $db->getQuery(true);
		$query
			->select($db->quoteName('a.extension_id'))
			->from($db->quoteName('#__extensions', 'a'))
			->where($db->quoteName('a.element') . ' = ' . $db->quote($element))
			->where($db->quoteName('a.folder') . ' = ' . $db->quote($folder))
			->where($db->quoteName('a.type') . ' = ' . $db->quote($type));

		$db->setQuery($query);
		$db->execute();
		if ($db->getNumRows()) {
			return $db->loadResult();
		}
		return false;
	}

	function buildLayoutPath($layout)
	{
		if (empty($layout)) $layout = "default";
		$core_file 	= dirname(__FILE__) . '/' . $this->_name . '/tmpl/' . $layout . '.php';
		return $core_file;
	}

	function buildLayout($vars, $layout = 'default')
	{
		ob_start();
		$layout = $this->buildLayoutPath($layout);
		include($layout);
		$html = ob_get_contents();
		ob_end_clean();
		return $html;
	}

	public function generateAuthenticationEnvelope($pub_key, $terminalID, $password, $amount)
	{
		$data = $terminalID . $password . str_pad($amount, 12, '0', STR_PAD_LEFT) . '00';
		$data = hex2bin($data);
		$AESSecretKey = openssl_random_pseudo_bytes(16);
		$ivlen = openssl_cipher_iv_length($cipher = "AES-128-CBC");
		$iv = openssl_random_pseudo_bytes($ivlen);
		$ciphertext_raw = openssl_encrypt($data, $cipher, $AESSecretKey, $options = OPENSSL_RAW_DATA, $iv);
		$hmac = hash('sha256', $ciphertext_raw, true);
		$crypttext = '';

		openssl_public_encrypt($AESSecretKey . $hmac, $crypttext, $pub_key);

		return array(
			"data" => bin2hex($crypttext),
			"iv" => bin2hex($iv),
		);
	}

	function onDigicom_PayGetHTML($vars, $pg_plugin)
	{
		if ($pg_plugin != $this->_name) return;
		$vars->custom_name = $this->params->get('plugin_name');
		$configs = JComponentHelper::getComponent('com_digicom')->params;
		//$vars->custom_email=$this->params->get( 'plugin_mail' );
		$price = intval(str_replace(".", " ", plgDigiCom_PaydigiirankishHelper::getPayerPrice($vars->order_id)));
		//=========================================================
		$app	= JFactory::getApplication();

		$currency = $configs->get('currency');

		$Amount = round($price, 0);

		if ($currency == 'IRT') {

			$Amount = round($price, 0) * 10;
		}


		$Description = 'خرید محصول از فروشگاه   ' . $configs->get('store_name', 'DigiCom Store');
		$Email = '';
		$Mobile = '';
		$CallbackURL = $vars->return;

		$factornumber = rand();
		$factornumber = substr($factornumber, '0', '6');
		$pubKey = $this->params->get('pub_key');
		$terminalID = $this->params->get('terminalID');
		$password = $this->params->get('password');
		$acceptorId = $this->params->get('acceptorId');


		$token = $this->generateAuthenticationEnvelope($pubKey, $terminalID, $password, $Amount);

		$data = array();
		$data["request"] = array(
			"acceptorId" => $acceptorId,
			"amount" => (int) $Amount,
			"billInfo" => null,
			"requestId" => uniqid(),
			"paymentId" => (string) $vars->order_id,
			"requestTimestamp" => time(),
			"revertUri" => $CallbackURL,
			"terminalId" => $terminalID,
			"transactionType" => "Purchase"
		);
		$data['authenticationEnvelope'] = $token;
		$data_string = json_encode($data);
		$ch = curl_init('https://ikc.shaparak.ir/api/v3/tokenization/make');
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
		curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
			'Content-Type: application/json',
			'Content-Length: ' . strlen($data_string)
		));

		$result = curl_exec($ch);
		$response = json_decode($result, JSON_OBJECT_AS_ARRAY);

		if ($response["responseCode"] != "00") {
			echo plgDigiCom_PaydigiirankishHelper::Fault_IranKish($response["responseCode"]);
			exit;
		}

?>

		<form id="kicapeyment" action="https://ikc.shaparak.ir/iuiv3/IPG/Index/" method="POST">
			<input type="hidden" name="tokenIdentity" value="<?php echo $response['result']['token'] ?>">
		</form>
		<script>
			document.forms["kicapeyment"].submit()
		</script>


<?php	}

	function onDigicom_PayGetInfo($config)
	{
		if (!in_array($this->_name, $config)) return;
		$obj 		= new stdClass;
		$obj->name 	= $this->params->get('plugin_name');
		$obj->id	= $this->_name;
		return $obj;
	}

	function onDigicom_PayProcesspayment($data)
	{


		$processor = JFactory::getApplication()->input->get('processor', '');
		if ($processor != $this->_name) return;
		$app	= JFactory::getApplication();
		$jinput = $app->input;
		$orderId = $jinput->get->get('order_id', '0', 'INT');
		$price = intval(str_replace(".", " ", plgDigiCom_PaydigiirankishHelper::getPayerPrice($orderId)));
		$trackingCode = "";
		//========================================================================

		$configs = JComponentHelper::getComponent('com_digicom')->params;

		$currency = $configs->get('currency');
		$Amount = round($price, 0);
		if ($currency == 'IRT') {

			$Amount = round($price, 0) * 10;
		}

		$terminalID = $this->params->get('terminalID');




		$referenceId = trim($_POST['retrievalReferenceNumber']); // 

		if ($_POST['responseCode'] == 0 || $_POST['responseCode'] == '00') {
			$data = array(
				"terminalId" => $terminalID,
				"retrievalReferenceNumber" => $_POST['retrievalReferenceNumber'],
				"systemTraceAuditNumber" => $_POST['systemTraceAuditNumber'],
				"tokenIdentity" => $_POST['token'],
			);

			$data_string = json_encode($data);
			$ch = curl_init('https://ikc.shaparak.ir/api/v3/confirmation/purchase');
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_HTTPHEADER, array(
				'Content-Type: application/json',
				'Content-Length: ' . strlen($data_string)
			));

			$result = curl_exec($ch);

			curl_close($ch);

			$response = json_decode($result, JSON_OBJECT_AS_ARRAY);

			if ($response["responseCode"] == 0 || $response["responseCode"] == "00") {
				$payment_status = $this->translateResponse('Completed');
				$db = JFactory::getDbo();
				$query = $db->getQuery(true);
				$fields = array($db->qn('amount_paid') . ' = ' . $db->q($price));
				$conditions = array($db->qn('id') . ' = ' . $db->q(intval($orderId)));
				$query->update($db->qn('#__digicom_orders'))->set($fields)->where($conditions);
				$db->setQuery((string) $query);
				$db->execute();
				$trackingCode = $referenceId;

				$msg = "با موفقیت پرداخت شد";
				$message = "کد پیگیری" . $trackingCode;
				$app->enqueueMessage($msg . '<br/>' . $message, 'message');
			} else {
				$msg = plgDigiCom_PaydigiirankishHelper::Fault_IranKish($response["responseCode"]);
				$app->enqueueMessage($msg, 'error');
				$payment_status = $this->translateResponse('Pending');
			}
		} else {
			$msg = plgDigiCom_PaydigiirankishHelper::Fault_IranKish($_POST['responseCode']);
			$app->enqueueMessage($msg, 'error');
			$payment_status = $this->translateResponse('Pending');
		}

		$data['payment_status'] = $payment_status;
		if (!isset($data['payment_status'])) {
			$info = array('raw_data'	=>	$data);
			$this->onDigicom_PayStorelog($this->_name, $info);
		}

		$result = array(
			'transaction_id'	=>	$this->getUniqueTransactionId($orderId),
			'order_id'				=>	$orderId,
			'status'					=>	$payment_status,
			'raw_data'				=>	json_encode($data),
			'trackingcode'			=>	"کد پیگیری" . $trackingCode,
			'processor'				=>	$processor
		);
		return $result;
	}

	function translateResponse($invoice_status)
	{

		foreach ($this->responseStatus as $key => $value) {
			if ($key == $invoice_status)
				return $value;
		}
	}

	function onDigicom_PayStorelog($name, $data)
	{
		if ($name != $this->_name) return;
		plgDigiCom_PaydigiirankishHelper::Storelog($this->_name, $data);
	}

	function getUniqueTransactionId($order_id)
	{
		$uniqueValue = $order_id . time();
		$long = md5(uniqid($uniqueValue, true));
		return substr($long, 0, 15);
	}
}
